from os import remove
import shlex
from os.path import isfile, join, split, splitext
from prody.tests import TestCase, skipIf, skipUnless

from numpy.testing import *

from prody.utilities import importDec
dec = importDec()

from prody.tests.datafiles import TEMPDIR, pathDatafile

from prody.apps import prody_parser

from prody.tests import MATPLOTLIB, NOPRODYCMD, WINDOWS

class TestPCACommand(TestCase):

    def setUp(self):

        self.command = ('pca --pdb {pdb} '
                        '-e -r -o {outdir} -v -z -t all -j '
                        '-f %8g -d , -x .dat '
                        '-R -Q -J 1,2 '
                        '-F png -D 120 -W 5 -H 4 ').format(outdir=TEMPDIR,
                        pdb=pathDatafile('multi_model_truncated'))
        self.suffixes = [
            '_pca_cc.png',
            '_pca.pca.npz',
            '_pca_covariance.dat',
            '_pca_cross-correlations.dat',
            '_pca_proj_1_2.png',
            '_pca_evalues.dat',
            '_pca_proj.dat',
            '_pca_evectors.dat',
            '_pca_sf.png',
            '_pca_extended_all.nmd',
            '_pca.nmd',
        ]

        self.tearDown()

    @dec.slow
    @skipIf(NOPRODYCMD, 'prody command not found')
    @skipUnless(MATPLOTLIB, 'matplotlib not found')
    @skipIf(WINDOWS, 'command tests are not run on Windows')
    def testPCACommandDCD(self):

        dcd = pathDatafile('dcd')
        command = self.command + dcd
        prefix = splitext(split(dcd)[1])[0]

        namespace = prody_parser.parse_args(shlex.split(command))
        namespace.func(namespace)

        for suffix in self.suffixes:
            fn = join(TEMPDIR, prefix + suffix)
            self.assertTrue(isfile(fn), msg=fn+' not found')

    @dec.slow
    @skipIf(NOPRODYCMD, 'prody command not found')
    @skipUnless(MATPLOTLIB, 'matplotlib not found')
    @skipIf(WINDOWS, 'command tests are not run on Windows')
    def testPCACommandPDB(self):

        dcd = pathDatafile('multi_model_truncated')
        command = self.command + dcd
        prefix = splitext(split(dcd)[1])[0]

        namespace = prody_parser.parse_args(shlex.split(command))
        namespace.func(namespace)

        for suffix in self.suffixes:
            fn = join(TEMPDIR, prefix + suffix)
            self.assertTrue(isfile(fn), msg=fn+' not found')


    def tearDown(self):

        for dcd in [pathDatafile('multi_model_truncated'),
                    pathDatafile('dcd')]:
            prefix = splitext(split(dcd)[1])[0]
            for suffix in self.suffixes:
                fn = join(TEMPDIR, prefix + suffix)
                if isfile(fn): remove(fn)



"""This module defines class that can be used a package wide logger."""

import sys
import math
import time
import os.path
import logging
import datetime
import logging.handlers
import numbers

__all__ = ['PackageLogger', 'LOGGING_LEVELS', 'LOGGER']

LOGGING_PROGRESS = logging.INFO + 5

LOGGING_LEVELS = {'debug': logging.DEBUG,
                'info': logging.INFO,
                'progress': LOGGING_PROGRESS,
                'warning': logging.WARNING,
                'error': logging.ERROR,
                'critical': logging.CRITICAL,
                'none': logging.CRITICAL}
LOGGING_INVERSE = {}
for key, value in LOGGING_LEVELS.items(): # PY3K: OK
    LOGGING_INVERSE[value] = key

now = datetime.datetime.now

class PackageLogger(object):

    """A class for package wide logging functionality."""

    def __init__(self, name, **kwargs):
        """Start logger for the package. Returns a logger instance.

        :arg prefix: prefix to console log messages, default is ``'@> '``
        :arg console: log level for console (``sys.stderr``) messages,
            default is ``'debug'``
        :arg info: prefix to log messages at *info* level
        :arg warning: prefix to log messages at *warning* level, default is
            ``'WARNING '``
        :arg error: prefix to log messages at *error* level, default is
            ``'ERROR '``
        """

        self._level = logging.DEBUG
        self._logger = logger = logging.getLogger(name)
        logger.setLevel(self._level)


        for handler in logger.handlers:
            handler.close()
        logger.handlers = []

        console = logging.StreamHandler()
        console.setLevel(LOGGING_LEVELS[kwargs.get('console', 'debug')])
        logger.addHandler(console)
        self.prefix = kwargs.get('prefix', '@> ')

        self._info = kwargs.get('info', '')
        self._warning = kwargs.get('warning', 'WARNING ')
        self._error = kwargs.get('error', 'ERROR ')

        self._n = None
        self._barlen = None
        self._line = None
        self._times = {}
        self._info = {}

        self._n_progress = 0

    # ====================
    # Attributes
    # ====================

    def _getverbosity(self):

        return LOGGING_INVERSE.get(self._logger.handlers[0].level)

    def _setverbosity(self, level):
        lvl = LOGGING_LEVELS.get(str(level).lower(), None)
        if lvl is None:
            self.warn('{0} is not a valid log level.'.format(level))
        else:
            self._logger.handlers[0].level = lvl
            self._level = lvl

    verbosity = property(_getverbosity, _setverbosity, doc=
        """Verbosity *level* of the logger, default level is **debug**.  Log
        messages are written to ``sys.stderr``.  Following logging levers are
        recognized:

        ========  =============================================
        Level     Description
        ========  =============================================
        debug     Everything will be printed to the sys.stderr.
        info      Only brief information will be printed.
        warning   Only warning messages will be printed.
        none      Nothing will be printed.
        ========  =============================================""")

    def _getprefix(self):

        return self._prefix

    def _setprefix(self, prefix):

        self._prefix = str(prefix)
        prefix += '%(message)s'
        self._logger.handlers[0].setFormatter(logging.Formatter(prefix))

    prefix = property(_getprefix, _setprefix, doc='String prepended to console'
                      ' log messages.')

    # ====================
    # Logging methods
    # ====================

    def info(self, msg):
        """Log *msg* with severity 'INFO'."""

        self.clear()
        self._logger.info(msg)

    def critical(self, msg):
        """Log *msg* with severity 'CRITICAL'."""

        self.clear()
        self._logger.critical(msg)

    def debug(self, msg):
        """Log *msg* with severity 'DEBUG'."""

        self.clear()
        self._logger.debug(msg)

    def warning(self, msg):
        """Log *msg* with severity 'WARNING'."""

        self.clear()
        self._logger.warning(self._warning + msg)

    warn = warning

    def error(self, msg):
        """Log *msg* with severity 'ERROR' and terminate with status 2."""

        self.clear()
        self._logger.error(self._error + msg)
        self.exit(2)

    def write(self, line):
        """Write *line* into ``sys.stderr``."""

        self._line = str(line)
        if self._level < logging.WARNING:
            sys.stderr.write(self._line)
            sys.stderr.flush()

    def clear(self):
        """Clear current line in ``sys.stderr``."""

        if self._level != LOGGING_PROGRESS: 
            if self._line and self._level < logging.WARNING:
                sys.stderr.write('\r' + ' ' * (len(self._line)) + '\r')
                self._line = ''

    def exit(self, status=0):
        """Exit the interpreter."""

        sys.exit(status)

    # ====================
    # Handlers & logfiles
    # ====================

    def addHandler(self, hdlr):
        """Add the specified handler to this logger."""

        self._logger.addHandler(hdlr)

    def getHandlers(self):
        """Returns handlers."""

        return self._logger.handlers

    def delHandler(self, index):
        """Remove handler at given *index* from the logger instance."""

        self._logger.handlers.pop(index)

    def start(self, filename, **kwargs):
        """Start a logfile.  If *filename* does not have an extension.
        :file:`.log` will be appended to it.

        :arg filename: name of the logfile
        :arg mode: mode in which logfile will be opened, default is "w"
        :arg backupcount: number of existing *filename.log* files to
            backup, default is 1"""

        filename = str(filename)
        if os.path.splitext(filename)[1] == '':
            filename += '.log'
        rollover = False
        # if mode='a' is provided, rollover is not performed
        if os.path.isfile(filename) and kwargs.get('mode', None) != 'a':
            rollover = True
        logfile = logging.handlers.RotatingFileHandler(filename,
                    mode=kwargs.get('mode', 'a'), maxBytes=0,
                    backupCount=kwargs.get('backupcount', 1))
        logfile.setLevel(LOGGING_LEVELS[kwargs.get('loglevel', 'debug')])
        logfile.setFormatter(logging.Formatter('%(message)s'))
        self.info("Logging into file: {0}".format(filename))
        self._logger.addHandler(logfile)
        if rollover:
            logfile.doRollover()
        self.info("Logging started at {0}".format(str(now())))

    def close(self, filename):
        """Close logfile *filename*."""

        filename = str(filename)
        if os.path.splitext(filename)[1] == '':
            filename += '.log'
        for index, handler in enumerate(self.getHandlers()):
            if isinstance(handler, logging.handlers.RotatingFileHandler):
                if handler.stream.name in (filename,os.path.abspath(filename)):
                    self.info("Logging stopped at {0}".format(str(now())))
                    handler.close()
                    self.delHandler(index)
                    self.info("Closing logfile: {0}".format(filename))
                    return
        self.warning("Logfile '{0}' was not found.".format(filename))

    # ====================
    # Progress and timing
    # ====================

    def progress(self, msg, steps, label=None, **kwargs):
        """Instantiate a labeled process with message and number of steps."""

        if steps is not None:  # if None then no upperlimit
            assert isinstance(steps, numbers.Integral) and steps > 0, \
                'steps must be a positive integer'
        
        self._times[label] = time.time()
        self._info[label] = {}
        self._info[label]['steps'] = steps
        self._info[label]['msg'] = msg
        self._info[label]['last'] = 0

        if not hasattr(self, '_verb'):
            self._verb = self._getverbosity()
            if self._level < logging.WARNING:
                self._setverbosity('progress')
        self._n_progress += 1

    def update(self, step, msg=None, label=None):
        """Update progress status to current line in the console."""

        assert isinstance(step, numbers.Integral), 'step must be a positive integer'
        
        if msg is None:
            msg = self._info[label]['msg']
        else:
            self._info[label]['msg'] = msg
        
        last = self._info[label]['last']
        n = self._info[label]['steps']
        i = step
        if self._level < logging.WARNING:
            start = self._times[label]
            sys.stderr.write('\r' + ' ' * last + '\r')
            if n is None:  # no upperlimit
                line = self._prefix + msg % i
            elif i <= n:
                percent = 100 * i / n
                if percent > 3:
                    seconds = int(math.ceil((time.time()-start) * (n-i)/i))
                    line = self._prefix + msg + ' [%3d%%] %ds' % (percent, seconds)
                else:
                    line = self._prefix + msg + ' [%3d%%]' % percent
            else:
                return
            sys.stderr.write(line)
            sys.stderr.flush()
            self._line = line
            self._info[label]['last'] = len(line)

    def finish(self):
        self._n_progress -= 1
        if self._n_progress < 0:
            self._n_progress = 0
        if self._n_progress == 0:
            if hasattr(self, '_verb'):
                self._setverbosity(self._verb)
                del self._verb
                self.clear()

    def sleep(self, seconds, msg=''):
        """Sleep for seconds while updating screen message every second.
        Message will start with ``'Waiting for Xs '`` followed by *msg*."""

        msg = str(msg)
        for second in range(int(seconds), 0, -1):
            self.write('Waiting for {0}s {1}'.format(second, msg))
            time.sleep(1)
            self.clear()

    def timeit(self, label=None):
        """Start timing a process.  Use :meth:`timing` and :meth:`report` to
        learn and report timing, respectively."""

        self._times[label] = time.time()

    def timing(self, label=None):
        """Returns timing for a labeled or default (**None**) process."""

        return time.time() - self._times.get(label, 0)

    def report(self, msg='Completed in %.2fs.', label=None):
        """Write *msg* with timing information for a labeled or default process
        at *debug* logging level."""

        self.debug(msg % (time.time() - self._times[label]))

LOGGER = PackageLogger('.prody')



# -*- coding: utf-8 -*-
"""This module defines :class:`HierView` class that builds a hierarchical
views of atom groups."""

from numpy import unique, zeros, arange, concatenate
from prody.utilities.misctools import count

from .atomgroup import AtomGroup
from .selection import Selection
from .chain import Chain
from .atommap import AtomMap
from .residue import Residue
from .segment import Segment

__all__ = ['HierView']


class HierView(object):

    """Hierarchical views can be generated for :class:`.AtomGroup`,
    :class:`.Selection`, and :class:`.Chain` instances. Indexing a 
    :class:`HierView` instance returns a :class:`.Chain` instance.

    Some :class:`object` methods are customized as follows:

    * :func:`len` returns the number of chains, i.e. :meth:`numChains`
    * :func:`iter` yields :class:`.Chain` instances
    * indexing by:
         - *segment name* (:func:`str`), e.g. ``"PROT"``, returns
           a :class:`.Segment`
         - *chain identifier* (:func:`str`), e.g. ``"A"``, returns
           a :class:`.Chain`
         - *[segment name,] chain identifier, residue number[, insertion code]*
           (:func:`tuple`), e.g. ``"A", 10`` or  ``"A", 10, "B"`` or
           ``"PROT", "A", 10, "B"``, returns a :class:`.Residue`

    Note that when an :class:`.AtomGroup` instance have distinct segments,
    they will be considered when building the hierarchical view.
    A :class:`.Segment` instance will be generated for each distinct segment
    name.  Then, for each segment chains and residues will be evaluated.
    Having segments in the structure will not change most behaviors of this
    class, except indexing.  For example, when indexing a hierarchical view
    for chain P in segment PROT needs to be indexed as ``hv['PROT', 'P']``."""

    def __init__(self, atoms, **kwargs):

        if not isinstance(atoms, (AtomGroup, Selection, Chain, Segment, AtomMap)):
            raise TypeError('atoms must be an AtomGroup, Selection, Chain, AtomMap or Segment instance')

        self._atoms = atoms
        self.update(**kwargs)

    def __repr__(self):

        if self._segments:
            return ('<HierView: {0} ({1} segments, {2} chains, {3} '
                    'residues)>').format(str(self._atoms), self.numSegments(),
                                         self.numChains(), self.numResidues())
        else:
            return ('<HierView: {0} ({1} chains, {2} residues)>'
               ).format(str(self._atoms), self.numChains(), self.numResidues())

    def __str__(self):

        return 'HierView of {0}'.format(str(self._atoms))

    def __len__(self):

        return len(self._chains)

    def __getitem__(self, key):

        if isinstance(key, str):
            return self.getSegment(key) or self.getChain(key)

        elif isinstance(key, tuple):
            length = len(key)

            if length == 1:
                return self.__getitem__(key[0])

            if length == 2:
                return (self.getChain(key[1], key[0]) or
                         self.getResidue(*key) or
                         self.getResidue(None, key[0], key[1]))

            if length == 3:
                return self.getResidue(*key) or self.getResidue(key[1],
                                                key[2], None, key[0])

            if length == 4:
                return self.getResidue(key[1], key[2], key[3], key[0])

        elif isinstance(key, int):
            return self.getResidue(None, key)

    def _getSegname(self):
        """Returns name of the segment when there is only one segment."""

        if self.numSegments() == 1:
            return self._ag._getSegnames()[0]

    def _getChid(self):
        """Returns identifier of the chain when there is only one chain."""

        if self.numChains() == 1:
            return self._ag._getChids()[0]

    def _getResidue(self, index):

        try:
            residue = self._residues[index]
        except IndexError:
            pass
        else:
            if residue is not None:
                try:
                    residue.getAtomGroup()
                except AttributeError:
                    residue = self._residues[index] = Residue(self._ag,
                                            residue, self, self._acsi,
                                            unique=True, selstr=self._selstr)
            return residue

    def _getChain(self, index):

        try:
            chain = self._chains[index]
        except IndexError:
            pass
        else:
            if chain is not None:
                try:
                    chain.getAtomGroup()
                except AttributeError:
                    chain = self._chains[index] = Chain(self._ag,
                                            chain, self, self._acsi,
                                            unique=True, selstr=self._selstr)
            return chain

    def _getSegment(self, index):

        try:
            segment = self._segments[index]
        except IndexError:
            pass
        else:
            if segment is not None:
                try:
                    segment.getAtomGroup()
                except AttributeError:
                    segment = self._segments[index] = Segment(self._ag,
                                            segment, self, self._acsi,
                                            unique=True, selstr=self._selstr)
            return segment

    def getAtoms(self):
        """Returns atoms for which the hierarchical view was built."""

        return self._atoms

    def update(self, **kwargs):
        """Update (or build) hierarchical view of atoms.  This method is called
        at instantiation, but can be used to rebuild the hierarchical view when
        attributes of atoms change."""

        self._acsi = self._atoms.getACSIndex()
        try:
            self._ag = self._atoms.getAtomGroup()
        except AttributeError:
            self._selstr = None
            self._update(**kwargs)
        else:
            self._selhv(**kwargs)

    def _selhv(self, **kwargs):
        """Build hierarchical view for :class:`.Selection` instances."""

        atoms = self._atoms
        ag = self._ag
        indices = atoms._getIndices()
        self._selstr = atoms.getSelstr()

        self._dict = ag.getHierView()._dict

        self._segments = _segments = [None] * ag.numSegments()
        self._residues = _residues = [None] * ag.numResidues()
        self._chains = _chains = [None] * ag.numChains()

        for hvidx, _list in [(atoms._getSegindices(), _segments),
                             (atoms._getChindices(), _chains),
                             (atoms._getResindices(), _residues),]:
            if not _list: continue
            pidx = hvidx[0]
            pi = 0
            for i, idx in enumerate(hvidx):
                if pidx == idx: continue
                subset = _list[pidx]
                if subset is None:
                    _list[pidx] = indices[pi:i]
                else:
                    _list[pidx] = concatenate((subset, indices[pi:i]))
                pidx, pi = idx, i
            subset = _list[pidx]
            if subset is None:
                _list[pidx] = indices[pi:]
            else:
                _list[pidx] = concatenate((subset, indices[pi:]))

    def _update(self, **kwargs):
        """Build hierarchical view for :class:`.AtomGroup` instances."""

        ag = self._ag = self._atoms
        n_atoms = len(ag)
        _indices = arange(n_atoms)

        self._dict = _dict = {}
        self._residues = _residues = []
        self._segments = _segments = []
        self._chains = _chains = []

        nones = None
        getnones = lambda: [None] * n_atoms if nones is None else nones
        termini = ag.getFlags('pdbter')
        if termini is None:
            termini = getnones()

        # identify segments
        segindex = -1
        segindices = zeros(n_atoms, int)

        sgnms = ag._getSegnames()
        if sgnms is None:
            _segments = None
        else:
            s = sgnms[0]
            if len(unique(sgnms)) == 1:
                # 1 segment
                if s:
                    _segments.append(_indices)
                    _dict[s] = 0
                else:
                    _segments = None
            else:
                ps = None       # previous segment name
                for i, s in enumerate(sgnms):
                    if s == ps or s in _dict:
                        continue
                    ps = s
                    idx = _indices[i:][sgnms[i:] == s]
                    segindex += 1
                    segindices[idx] = segindex
                    _dict[s or None] = segindex
                    _segments.append(idx)

        ag._data['segindex'] = segindices

        # identify chains
        chindex = -1
        chindices = zeros(n_atoms, int)

        chids = ag._getChids()
        if chids is None:
            _chains = None
        else:
            if _segments is None:
                if len(unique(chids)) == 1:
                    _dict[(None, chids[0] or None)] = 0
                    _chains.append(_indices)
                else:
                    pc = None
                    for i, c in enumerate(chids):
                        if c == pc or (None, c) in _dict:
                            continue
                        pc = c
                        idx = _indices[i:][chids[i:] == c]
                        chindex += 1
                        chindices[idx] = chindex
                        _dict[(None, c)] = chindex
                        _chains.append(idx)
            else:
                pc = chids[0]
                ps = sgnms[0]
                _i = 0
                for i, c in enumerate(chids):
                    s = sgnms[i]
                    if c == pc and s == ps:
                        continue
                    s_c = (ps, pc or None)
                    cid = _dict.get(s_c)
                    idx = _indices[_i:i]
                    if cid is None:
                        #segment = _dict[ps]
                        chindex += 1
                        chindices[idx] = chindex
                        _dict[s_c] = chindex
                        _chains.append(idx)
                    else:
                        chain = _chains[cid]
                        chindices[idx] = cid
                        _chains[cid] = concatenate((chain, idx))
                    pc = c
                    ps = s
                    _i = i
                s_c = (ps or None, pc or None)
                cid = _dict.get(s_c)
                idx = _indices[_i:]
                if cid is None:
                    #segment = _dict[ps]
                    chindex += 1
                    chindices[idx] = chindex
                    _dict[s_c] = chindex
                    _chains.append(idx)
                else:
                    chain = _chains[cid]
                    chindices[idx] = cid
                    _chains[cid] = concatenate((chain, idx))

        ag._data['chindex'] = chindices

        if kwargs.get('chain') == True:
            return

        # identify residues
        resindex = -1
        resindices = zeros(n_atoms, int)

        rnums = ag._getResnums()
        if rnums is None:
            raise ValueError('resnums are not set')
        if _segments is None:
            sgnms = nones = getnones()
        if _chains is None:
            chids = nones = getnones()
        icods = ag._getIcodes()
        if icods is None:
            icods = nones = getnones()
        pr = rnums[0]
        pi = icods[0] or None
        pc = chids[0]
        ps = sgnms[0]
        _j = 0
        _append = _residues.append
        _get = _dict.get
        _set = _dict.__setitem__
        for j, r in enumerate(rnums):
            i = icods[j] or None
            c = chids[j]
            s = sgnms[j]
            if r != pr or i != pi or c != pc or s != ps or (j and termini[j-1]):
                s_c_r_i = (ps, pc, pr, pi)
                rid = _get(s_c_r_i)
                idx = _indices[_j:j]
                if (rid is None or isinstance(rid, list) or
                    termini[_residues[rid][-1]]):
                    resindex += 1
                    resindices[idx] = resindex
                    _append(idx)
                    if rid is None:
                        _set(s_c_r_i, resindex)
                    elif isinstance(rid, list):
                        rid.append(resindex)
                    else:
                        _set(s_c_r_i, [rid, resindex])
                else:
                    residue = _residues[rid]
                    resindices[idx] = rid
                    _residues[rid] = concatenate((residue, idx))
                ps = s
                pc = c
                pr = r
                pi = i
                _j = j
        s_c_r_i = (ps, pc, pr, pi)
        rid = _get(s_c_r_i)
        idx = _indices[_j:]
        if rid is None or isinstance(rid, list) or termini[_residues[rid][-1]]:
            resindex += 1
            resindices[idx] = resindex
            _append(idx)
            if rid is None:
                _set(s_c_r_i, resindex)
            elif isinstance(rid, list):
                rid.append(resindex)
            else:
                _set(s_c_r_i, [rid, resindex])
        else:
            residue = _residues[rid]
            resindices[idx] = rid
            _residues[rid] = concatenate((residue, idx))

        ag._data['resindex'] = resindices

    def getResidue(self, chid, resnum, icode=None, segname=None):
        """Returns residue with number *resnum* and insertion code *icode* from
        the chain with identifier *chid* in segment with name *segname*."""

        try:
            index = self._dict[(segname or self._getSegname(),
                                chid or self._getChid(),
                                resnum, icode or None)]
        except KeyError:
            pass
        else:
            if isinstance(index, list):
                return [r for r in [self._getResidue(i) for i in index]
                        if r is not None]
            else:
                return self._getResidue(index)

    def numResidues(self):
        """Returns number of residues."""

        return (len(self._residues) if self._ag is self._atoms else
                len(self._residues) - count(self._residues, None))

    def iterResidues(self):
        """Yield residues."""

        alist = self._residues
        ag = self._ag
        acsi = self._acsi
        selstr = self._selstr
        for i, item in enumerate(alist):
            if item is None:
                continue
            try:
                item.dtype
            except AttributeError:
                pass
            else:
                item = alist[i] = Residue(ag, item, self, acsi, selstr=selstr,
                                          unique=True)
            yield item

    def getChain(self, chid, segname=None):
        """Returns chain with identifier *chid*, if it is present."""

        try:
            index = self._dict[(segname or self._getSegname() or None,
                                chid or None)]
        except KeyError:
            try:
                index = None
                for key in self._dict.keys():
                    if isinstance(key, tuple) and len(key) == 2:
                        if key[1] == 'BU':
                            index = self._dict[key]
                if index is None:
                    raise ValueError('invalid chain ID')
            except ValueError:
                pass
            else:
                return self._getChain(index)
        else:
            return self._getChain(index)

    def iterChains(self):
        """Yield chains."""

        alist = self._chains
        ag = self._ag
        acsi = self._acsi
        selstr = self._selstr
        for i, item in enumerate(alist):
            if item is None:
                continue
            try:
                item.dtype
            except AttributeError:
                pass
            else:
                item = alist[i] = Chain(ag, item, self, acsi, selstr=selstr,
                                        unique=True)
            yield item

    __iter__ = iterChains

    def numChains(self):
        """Returns number of chains."""

        return (len(self._chains) if self._ag is self._atoms else
                len(self._chains) - count(self._chains, None))

    def getSegment(self, segname):
        """Returns segment with name *segname*, if it is present."""

        try:
            index = self._dict[segname or None]
        except KeyError:
            pass
        else:
            return self._getSegment(index)

    def numSegments(self):
        """Returns number of chains."""

        return (len(self._segments) if self._ag is self._atoms else
                len(self._segments) - count(self._segments, None))

    def iterSegments(self):
        """Yield segments."""

        alist = self._segments
        ag = self._ag
        acsi = self._acsi
        selstr = self._selstr
        for i, item in enumerate(alist):
            if item is None:
                continue
            try:
                item.dtype
            except AttributeError:
                pass
            else:
                item = alist[i] = Segment(ag, item, self, acsi, selstr=selstr,
                                          unique=True)
            yield item



from prody import LOGGER
from prody.proteins import parsePDB
from prody.utilities import dictElement, dictElementLoop, openURL

import re
from xml.etree.cElementTree import XML

__all__ = ['UniprotRecord', 'searchUniprot', 'queryUniprot']

comma_splitter = re.compile(r'\s*,\s*').split

class UniprotRecord(object):
    """This class provides a wrapper for UniProt data including functions 
    for accessing particular fields and parsing associated PDB entries."""
    def __init__(self, data):
        self._rawdata = data
        self._pdbids = []
        # self._selstrs = []

        self._parse()

    def __repr__(self):
        return '<UniprotRecord: %s>'%self.getTitle()

    def __str__(self):
        return self.getTitle()

    def setData(self, value):
        self._rawdata = value
        self._parse()

    def getData(self):
        return self._rawdata

    def getPDBs(self):
        return self._pdbids

    # def getSelstrs(self):
    #     return self._selstrs

    def getSequence(self, index=0):
        return self.getEntry('sequence', index)
    
    def getAccession(self, index=0):
        return self.getEntry('accession', index)
    
    def getName(self, index=0):
        return self.getEntry('name', index)

    def getTitle(self):
        uid = self.getAccession()
        name = self.getName()
        return '%s (%s)'%(uid, name)

    def getEntry(self, item, index=0):
        key = '%s%4d'%(item, index)
        if key in self._rawdata:
            return self._rawdata[key]
        else:
            raise KeyError('%s does not exist in the Uniprot record'%key)

    def _parse(self):
        data = self._rawdata
        PDBIDs = []
        # SELSTRs = []
        for key, value in data.items():
            if not key.startswith('dbReference'):
                continue
            try:
                pdbid = value['PDB']
            except (KeyError, TypeError) as e:
                continue
            pdbchains = value['chains']
            
            # example chain strings: "A=27-139, B=140-150" or "A/B=27-150"
            chains = []
            ranges = []
            pdbchains = comma_splitter(pdbchains)
            for chain in pdbchains:
                chids, resrange = chain.split('=')
                chids = [chid.strip() for chid in chids.split('/')]
                resrange = resrange.split('-')

                for chid in chids:
                    chains.append(chid)
                    ranges.append(resrange)
            
            for chid, rng in zip(chains, ranges):
                pdbchid = pdbid + chid if chid != '@' else pdbid
                PDBIDs.append(pdbchid)
                # SELSTRs.append('resnum %s to %s'%tuple(rng))
        
        self._pdbids = PDBIDs
        # self._selstrs = SELSTRs

    def parsePDBs(self, **kwargs):
        """Load PDB into memory as :class:`.AtomGroup` instances using :func:`.parsePDB` and 
        perform selection based on residue ranges given by CATH."""
        
        pdbs = self.getPDBs()
        # selstrs = self.getSelstrs()
        header = kwargs.get('header', False)
        model = kwargs.get('model', None)

        LOGGER.timeit('_uniprot_parsePDB')
        LOGGER.info('Parsing {0} PDB files...'.format(len(pdbs)))
        ret = parsePDB(*pdbs, **kwargs)

        if model != 0:
            headers = None
            if header:
                prots, headers = ret
            else:
                prots = ret

            if not isinstance(prots, list):
                prots = [prots]

                if header:
                    headers = [headers]
                    ret = (prots, headers)
                else:
                    ret = prots
                    
            LOGGER.info('Extracting domains...')
            # for i in range(len(prots)):
            #     sel = prots[i].select(selstrs[i])
            #     prots[i] = sel
        LOGGER.report('Uniprot domains are parsed and extracted in %.2fs', '_uniprot_parsePDB')

        return ret

def queryUniprot(id, expand=[], regex=True):
    """Query Uniprot with *id* and return a `dict` containing the raw results. 
    Regular users should use :func:`searchUniprot` instead.
    
    :arg expand: entries through which you want to loop dictElements
        until there aren't any elements left
    :type expand: list
    """

    if not isinstance(id, str):
        raise TypeError('id should be a string')

    try:
        record_file = openURL('http://www.uniprot.org/uniprot/{0}.xml'.format(id))
    except:
        raise ValueError('No Uniprot record found with that id')
    
    data = record_file.read()
    record_file.close()
    data = XML(data)

    data = dictElement(data[0], '{http://uniprot.org/uniprot}', number_multiples=True)

    for key in data:
        value = data[key]
        if not key.startswith('dbReference'):
            continue
        
        try:
            if value.get('type') != 'PDB':
                continue
        except AttributeError:
            continue

        pdbid = value.get('id')
        refdata = {'PDB': pdbid}
        for prop in value:
            prop_key = prop.get('type')
            prop_val = prop.get('value')
            refdata[prop_key] = prop_val
        data[key] = refdata
            
    if expand:
        keys = []
        if regex:
            for lt in expand:
                lt_re = re.compile(lt)
                for key in data:
                    if lt_re.match(key):
                        keys.append(key)
        else:
            keys = expand
        data = dictElementLoop(data, keys, '{http://uniprot.org/uniprot}')
    
    return data

def searchUniprot(id):
    """Search Uniprot with *id* and return a :class:`UniprotRecord` containing the results. 
    """

    data = queryUniprot(id)
    
    return UniprotRecord(data)



# -*- coding: utf-8 -*-
"""This module defines a class for handling trajectory frames."""

import numpy as np

from prody.measure import getRMSD
from prody.utilities import importLA

__all__ = ['Frame']


class Frame(object):

    """A class for storing trajectory frame coordinates and provide methods
    acting on them."""

    __slots__ = ['_traj', '_index', '_coords', '_unitcell', '_velocs']

    def __init__(self, traj, index, coords, unitcell=None, velocs=None):

        self._traj = traj
        self._index = index

        self._coords = coords
        self._velocs = velocs
        self._unitcell = unitcell

    def __repr__(self):

        sel = ''
        if self._traj._indices is not None:
            sel = 'selected {0} of '.format(self.numSelected())

        return ('<Frame: {0} from {1} ({2}{3} atoms)>'
                ).format(self._index, self._traj.getTitle(), sel,
                         self._traj.numAtoms())

    def __str__(self):

        return 'Frame {0} from {1}'.format(self._index,
                                           self._traj.getTitle())

    def numAtoms(self):
        """Returns number of atoms."""

        return self._traj.numAtoms()

    def numSelected(self):
        """Returns number of selected atoms."""

        return self._traj.numSelected()

    def getAtoms(self):
        """Returns associated/selected atoms."""

        return self._traj.getAtoms()

    def getIndex(self):
        """Returns index."""

        return self._index

    def getWeights(self):
        """Returns coordinate weights for selected atoms."""

        return self._traj.getWeights()

    def _getWeights(self):

        return self._traj._getWeights()

    def getTrajectory(self):
        """Returns the trajectory."""

        return self._traj

    def getCoords(self):
        """Returns a copy of coordinates of (selected) atoms."""

        indices = self._traj._indices
        if indices is None:
            return self._getxyz().copy()
        else:
            return self._getxyz()[indices]

    def _getCoords(self):
        """Returns coordinates of (selected) atoms."""

        indices = self._traj._indices
        if indices is None:
            return self._getxyz()
        else:
            return self._getxyz()[indices]

    def _getxyz(self):
        """Returns coordinates array."""

        ag = self._traj.link()
        if ag is None:
            coords = self._coords
        else:
            coords = ag._getCoords()
        if coords is None:
            raise ValueError('coordinates are not set')
        return coords

    def getVelocities(self):
        """Returns a copy of velocities of (selected) atoms."""

        if self._velocs is not None:
            indices = self._traj._indices
            if indices is None:
                return self._velocs.copy()
            else:
                return self._velocs[indices]

    def _getVelocities(self):
        """Returns velocities of (selected) atoms."""

        if self._velocs is not None:
            indices = self._traj._indices
            if indices is None:
                return self._velocs
            else:
                return self._velocs[indices]

    def getUnitcell(self):
        """Returns a copy of unitcell array."""

        if self._unitcell is not None:
            return self._unitcell.copy()

    def _getUnitcell(self):

        return self._unitcell

    def getDeviations(self):
        """Returns deviations from the trajectory reference coordinates."""

        indices = self._traj._indices
        coords = self._getCoords()
        if indices is None:
            return coords - self._traj._coords
        else:
            return coords - self._traj._coords[indices]

    def getRMSD(self):
        """Returns RMSD from the trajectory reference coordinates.  If weights
        for the trajectory are set, weighted RMSD will be returned."""

        indices = self._traj._indices
        traj = self._traj
        coords = self._getCoords()
        if indices is None:
            return getRMSD(coords, traj._coords, traj._weights)
        else:
            if traj._weights is None:
                return getRMSD(coords, traj._coords[indices])
            else:
                return getRMSD(coords, traj._coords[indices],
                               traj._weights[indices])

    def superpose(self):
        """Superpose frame onto the trajectory reference coordinates.  Note
        that transformation matrix is calculated based on selected atoms and
        applied to all atoms. If atom weights for the trajectory are set, they
        will be used to calculate the transformation."""

        traj = self._traj
        indices = traj._indices
        mob = mov = self._getxyz()

        weights = traj._weights
        if indices is None:
            tar = traj._coords
            mov = None
        else:
            tar = traj._coords[indices]
            mob = mob[indices]
            if weights is not None:
                weights = weights[indices]

        linalg = importLA()
        if weights is None:
            mob_com = mob.mean(0)
            mob_org = mob - mob_com
            tar_com = tar.mean(0)
            tar_org = tar - tar_com
            matrix = np.dot(tar_org.T, mob_org)
        else:
            weights_sum = weights.sum()
            weights_dot = np.dot(weights.T, weights)
            mob_com = (mob * weights).sum(axis=0) / weights_sum
            mob_org = mob - mob_com
            tar_com = (tar * weights).sum(axis=0) / weights_sum
            tar_org = tar - tar_com
            matrix = np.dot((tar_org * weights).T,
                            (mob_org * weights)) / weights_dot

        U, s, Vh = linalg.svd(matrix)
        Id = np.array([[1, 0, 0],
                       [0, 1, 0],
                       [0, 0, np.sign(linalg.det(matrix))]])
        rotation = np.dot(Vh.T, np.dot(Id, U.T))

        if mov is None:
            np.add(np.dot(mob_org, rotation), tar_com, mob)
        else:
            np.add(np.dot(mov, rotation),
                   (tar_com - np.dot(mob_com, rotation)), mov)



# -*- coding: utf-8 -*-
"""This module defines a class for handling ensembles of conformations."""

from numbers import Integral

from numpy import dot, add, subtract, array, ndarray, sign, concatenate
from numpy import zeros, ones, arange, isscalar, max, asarray
from numpy import newaxis, unique, repeat, sum, empty, tile

from prody import LOGGER
from prody.atomic import Atomic, sliceAtoms
from prody.atomic.atomgroup import checkLabel
from prody.measure import getRMSD, calcDeformVector
from prody.utilities import importLA, checkCoords, checkWeights, copy, isListLike

from .conformation import *

__all__ = ['Ensemble']

class Ensemble(object):

    """A class for analysis of arbitrary conformational ensembles.

    Indexing (e.g. ``ens[0]``) returns a :class:`.Conformation` instance that
    points to a coordinate set in the ensemble.  Slicing (e.g. ``ens[0:10]``)
    returns an :class:`Ensemble` instance that contains a copy of the subset
    of conformations (coordinate sets). """

    def __init__(self, title='Unknown'):
        """Instantiate with a *title* or a :class:`.Atomic` instance.  All
        coordinate sets from atomic instances will be added to the ensemble."""

        self._title = str(title).strip()

        self._coords = None         # reference
        self._n_atoms = 0
        self._n_csets = 0  # number of conformations/frames/coordinate sets
        self._weights = None
        self._atoms = None
        self._indices = None  # indices of selected atoms

        self._confs = None       # coordinate sets
        self._data = dict()

        if isinstance(title, Ensemble):
            self._atoms = title.getAtoms()
        elif isinstance(title, Atomic):
            self._atoms = title

        if isinstance(title, (Atomic, Ensemble)):
            self.setCoords(title)
            self.addCoordset(title)

    def __repr__(self):

        if self._indices is None:
            return ('<{0}: {1} ({2} conformations; {3} atoms)>'
                    ).format(self.__class__.__name__, self.getTitle(), len(self), self._n_atoms)
        else:
            return ('<{0}: {1} ({2} conformations; selected {3} of '
                    '{4} atoms)>').format(self.__class__.__name__, self.getTitle(), len(self),
                                          self.numSelected(), self._n_atoms)

    def __str__(self):

        return 'Ensemble {0}'.format(self.getTitle())

    def __len__(self):

        return self._n_csets

    def __getitem__(self, index):
        """Returns a conformation at given index."""

        if self._confs is None:
            return None

        # use sth like follows
        # which = arange(self._n_csets)[index].nonzero()[0]
        # if len(which) == 1:
        #       return getConf...
        # else:
        #     return SubEnsemble

        if isinstance(index, Integral):
            return self.getConformation(index)

        elif isinstance(index, slice):
            ens = type(self)('{0} ({1[0]}:{1[1]}:{1[2]})'.format(
                                self.getTitle(), index.indices(len(self))))
            if self._coords is not None:
                ens.setCoords(self._coords.copy())
        
            ens.addCoordset(self._confs[index].copy())

            for key in self._data:
                ens._data[key] = self._data[key][index].copy()
            if self._weights is not None:
                ens.setWeights(self._weights.copy())
            
            ens.setAtoms(self._atoms)
            ens._indices = self._indices
            return ens

        elif isinstance(index, (list, ndarray)):
            ens = type(self)('{0}'.format(self.getTitle()))
            if self._coords is not None:
                ens.setCoords(self._coords.copy())
            ens.addCoordset(self._confs[index].copy())
            for key in self._data:
                ens._data[key] = self._data[key][index].copy()
            if self._weights is not None:
                ens.setWeights(self._weights.copy())

            ens.setAtoms(self._atoms)
            ens._indices = self._indices
            return ens

        else:
            raise IndexError('invalid index')

    def __add__(self, other):
        """Concatenate ensembles. The reference coordinates, atoms, 
        and weights of *self* is used in the resulting ensemble."""

        if not isinstance(other, Ensemble):
            raise TypeError('an Ensemble instance cannot be added to an {0} '
                            'instance'.format(type(other)))
        elif self._n_atoms != other._n_atoms:
            raise ValueError('Ensembles must have same number of atoms.')

        ensemble = type(self)('{0} + {1}'.format(self.getTitle(),
                                               other.getTitle()))
        if self._coords is not None:
            ensemble.setCoords(self._coords.copy())
        if self._confs is not None:
            ensemble.addCoordset(self._confs.copy())
        if other._confs is not None:
            ensemble.addCoordset(other._confs.copy())

        all_keys = set(list(self._data.keys()) + list(other._data.keys()))
        for key in all_keys:
            if key in self._data and key in other._data:
                self_data = self._data[key]
                other_data = other._data[key]
            elif key in self._data:
                self_data = self._data[key]
                other_data = zeros(other.numConfs(), dtype=self_data.dtype)
            elif key in other._data:
                other_data = other._data[key]
                self_data = zeros(other.numConfs(), dtype=other_data.dtype)
            ensemble._data[key] = concatenate((self_data, other_data), axis=0)

        if self._weights is not None:
            LOGGER.info('Atom weights from {0} are used in {1}.'
                        .format(repr(self._title), repr(ensemble.getTitle())))
            ensemble.setWeights(self._weights.copy())
        elif other._weights is not None:
            ensemble.setWeights(other._weights.copy())
        
        if self._atoms is not None:
            ensemble.setAtoms(self._atoms)
            ensemble._indices = self._indices
        else:
            ensemble.setAtoms(other._atoms)
            ensemble._indices = other._indices
        return ensemble

    def __iter__(self):

        n_csets = self._n_csets
        for i in range(n_csets):
            if n_csets != self._n_csets:
                raise RuntimeError('number of conformations in the ensemble '
                                   'changed during iteration')
            yield Conformation(self, i)

    def getTitle(self):
        """Returns title of the ensemble."""

        return self._title

    def setTitle(self, title):
        """Set title of the ensemble."""

        self._title = str(title)

    def numAtoms(self, selected=True):
        """Returns number of atoms."""
        if selected:
            return self.numSelected()
        return self._n_atoms

    def numConfs(self):
        """Returns number of conformations."""

        return self._n_csets

    numCoordsets = numConfs

    def numSelected(self):
        """Returns number of selected atoms.  Number of all atoms will be
        returned if a selection is not made.  A subset of atoms can be
        selected by passing a selection to :meth:`setAtoms`."""

        return self._n_atoms if self._indices is None else len(self._indices)

    def isSelected(self):
        """Returns if a subset of atoms are selected."""
        return self._indices is not None

    def getAtoms(self, selected=True):
        """Returns associated/selected atoms."""
        if self._atoms is None:
            return None
        if self._indices is None or not selected:
            return self._atoms
        return self._atoms[self._indices]

    def setAtoms(self, atoms):
        """Set *atoms* or specify a selection of atoms to be considered in
        calculations and coordinate requests.  When a selection is set,
        corresponding subset of coordinates will be considered in, for
        example, alignments and RMSD calculations.  Setting atoms also
        allows some functions to access atomic data when needed.  For
        example, :class:`.Ensemble` and :class:`.Conformation` instances
        become suitable arguments for :func:`.writePDB`.  Passing **None**
        as *atoms* argument will deselect atoms."""

        if atoms is None:
            self._atoms = self._indices = None
            return

        try:
            atoms.getACSIndex()
        except AttributeError:
            raise TypeError('atoms must be an Atomic instance')

        n_atoms = self._n_atoms
        if n_atoms:
            if atoms.numAtoms() > n_atoms:
                raise ValueError('atoms must be same size or smaller than '
                                'the ensemble')

            try:
                dummies = atoms.numDummies()
            except AttributeError:
                pass
            else:
                if not dummies:
                    indices = atoms._getIndices()
                    if any(indices != unique(indices)):
                        raise ValueError('atoms must be ordered by indices')

            if atoms.numAtoms() == n_atoms: # atoms is a complete set (AtomSubset can be a complete set)
                self._atoms = atoms
                self._indices = None

            else: # atoms is a subset
                try:
                    ag = atoms.getAtomGroup()
                except AttributeError:
                    ag = atoms
                if ag.numAtoms() != n_atoms:
                    raise ValueError('size mismatch between this ensemble ({0} atoms) and atoms ({1} atoms)'
                                    .format(n_atoms, ag.numAtoms()))
                self._atoms = ag
                self._indices, _ = sliceAtoms(self._atoms, atoms)
                
        else: # if assigning atoms to a new ensemble
            self._n_atoms = atoms.numAtoms()
            self._atoms = atoms
            self._indices = None

    def select(self, selection):
        """Selects columns corresponding to a part of the atoms."""

        atoms = self.getAtoms()
        if atoms is None:
            raise ValueError('to use select, atoms must be set for the ensemble')

        sel = atoms.select(selection)
        self.setAtoms(sel)

    def deselect(self):
        """Undoes the selection."""

        self._indices = None

    def getCoords(self, selected=True):
        """Returns a copy of reference coordinates for selected atoms."""

        if self._coords is None:
            return None
        if self._indices is None or not selected:
            return self._coords.copy()
        return self._coords[self._indices].copy()

    def getIndices(self):
        """Returns a copy of indices of selected columns"""

        return copy(self._indices)

    def setIndices(self, value):
        if not isListLike(value):
            raise TypeError('value must be a list or numpy.ndarray instance')

        array = asarray(value)

        if len(array) != self._n_atoms:
            raise ValueError('length mismatch between this ensemble '
                             '(%d) and indices (%d)'%(self._n_atoms, len(array)))
        self._indices = value

    def _getCoords(self, selected=True):
        """Returns a view of reference coordinates for selected atoms."""

        if self._coords is None:
            return None
        if self._indices is None or not selected:
            return self._coords
        return self._coords[self._indices].copy()

    def setCoords(self, coords):
        """Set *coords* as the ensemble reference coordinate set.  *coords*
        may be an array with suitable data type, shape, and dimensionality, or
        an object with :meth:`getCoords` method."""

        atoms = coords
        try:
            if isinstance(coords, Ensemble):
                coords = copy(coords._coords)
            else:
                coords = coords.getCoords()
        except AttributeError:
            pass
        finally:
            if coords is None:
                raise ValueError('coordinates of {0} are not set'
                                 .format(str(atoms)))

        try:
            checkCoords(coords, natoms=self._n_atoms)
        except TypeError:
            raise TypeError('coords must be a numpy array or an object '
                            'with `getCoords` method')

        self._coords = coords
        self._n_atoms = coords.shape[0]

        if isinstance(atoms, Ensemble):
            self._indices = atoms._indices
            self._atoms = atoms._atoms

    def getWeights(self, selected=True):
        """Returns a copy of weights of selected atoms."""

        if self._weights is None:
            return None
        if self._indices is None or not selected:
            return self._weights.copy()
        if self._weights.ndim == 2:
            return self._weights[self._indices].copy()
        else:
            return self._weights[:, self._indices].copy()

    def _getWeights(self, selected=True):

        if self._weights is None:
            return None
        if self._indices is None or not selected:
            return self._weights
        if self._weights.ndim == 2:
            return self._weights[self._indices]
        else:
            return self._weights[:, self._indices]

    def setWeights(self, weights):
        """Set atomic weights."""

        if weights is None:
            self._weights = None
        else:
            if self._n_atoms == 0:
                raise AttributeError('first set reference coordinates')
            try:
                self._weights = checkWeights(weights, self._n_atoms, None)
            except ValueError:
                weights = checkWeights(weights, self.numSelected(), None)
                if not self._weights:
                    self._weights = ones((self._n_atoms, 1), dtype=float)
                self._weights[self._indices, :] = weights    

    def addCoordset(self, coords, **kwargs):
        """Add coordinate set(s) to the ensemble.  
        
        :arg coords: must be a :class:`~numpy.ndarray` with suitable data type, 
                     shape and dimensionality, or an object with :meth:`getCoordsets` method. 
                     
        :kwarg data: associated data to be added along with *coords*.
        :type data: dict   
        """

        n_atoms = self._n_atoms
        n_select = self.numSelected()

        adddata = kwargs.pop('data', {})

        try:
            if self._coords is not None:
                if isinstance(coords, Ensemble):
                    coords = coords._getCoordsets(selected=False)
                elif hasattr(coords, '_getCoordsets'):
                    coords = coords._getCoordsets()
                elif hasattr(coords, '_getCoords'):
                    coords = coords._getCoords()
                    coords = array(coords)
            else:
                if isinstance(coords, Ensemble):
                    coords = coords.getCoordsets(selected=False)
                elif hasattr(coords, 'getCoordsets'):
                    coords = coords.getCoordsets()
                elif hasattr(coords, 'getCoords'):
                    coords = coords.getCoords()
                    coords = array(coords)

        except AttributeError:
            pass
        else:
            if coords is None:
                raise ValueError('coordinates are not set')

        try:
            checkCoords(coords, csets=True, natoms=n_atoms)
        except:
            try:
                checkCoords(coords, csets=True, natoms=n_select)
            except TypeError:
                raise TypeError('coords must be a numpy array or an object '
                                'with `getCoords` method')

        # update coordinates
        if coords.ndim == 2:
            n_nodes, _ = coords.shape
            coords = coords.reshape((1, n_nodes, 3))
            n_confs = 1
        else:
            n_confs, n_nodes, _ = coords.shape

        if not n_atoms:
            self._n_atoms = n_atoms = n_nodes

        if n_nodes == n_select and self.isSelected():
            full_coords = repeat(self._coords[newaxis, :, :], n_confs, axis=0)
            full_coords[:, self._indices, :] = coords
            coords = full_coords

        if self._confs is None:
            self._confs = coords
        else:
            self._confs = concatenate((self._confs, coords), axis=0)
        
        # appending new data
        if self._data is None:
            self._data = {}
        all_keys = set(list(self._data.keys()) + list(adddata.keys()))

        for key in all_keys:
            if key in self._data:
                data = self._data[key]
                if key not in adddata:
                    shape = [n_confs] 
                    for s in data.shape[1:]:
                        shape.append(s)
                    newdata = zeros(shape, dtype=data.dtype)
                else:
                    newdata = asarray(adddata[key])
                    if newdata.shape[0] != n_confs:
                        raise ValueError('the length of data["%s"] does not match that of coords'%key)
            else:
                newdata = asarray(adddata[key])
                shape = [self._n_csets]
                for s in newdata.shape[1:]:
                    shape.append(s)
                data = zeros(shape, dtype=newdata.dtype)
            self._data[key] = concatenate((data, newdata), axis=0)

        # update the number of coordinate sets
        self._n_csets += n_confs

    def getCoordsets(self, indices=None, selected=True):
        """Returns a copy of coordinate set(s) at given *indices*, which may be
        an integer, a list of integers or **None**. **None** returns all
        coordinate sets.  For reference coordinates, use :meth:`getCoords`
        method."""

        coords = self._getCoordsets(indices, selected)
        return copy(coords)

    def _getCoordsets(self, indices=None, selected=True):

        if self._confs is None:
            return None
        if self._indices is None or not selected:
            if indices is None:
                return self._confs
            try:
                return self._confs[indices]
            except IndexError:
                pass
        else:
            selids = self._indices
            if indices is None:
                return self._confs[:, selids]
            try:
                return self._confs[indices, selids]
            except IndexError:
                pass
        raise IndexError('indices must be an integer, a list/array of '
                         'integers, a slice, or None')

    def delCoordset(self, index):
        """Delete a coordinate set from the ensemble."""

        if isscalar(index):
            index = [index]
        else:
            index = list(index)
        length = self._n_csets
        which = ones(length, bool)
        which[index] = False
        if which.sum() == 0:
            self._confs = None
            self._weights = None
        else:
            self._confs = self._confs[which]
            if self._weights is not None:
                self._weights = self._weights[which]
        self._n_csets -= len(index)

    def iterCoordsets(self):
        """Iterate over coordinate sets. A copy of each coordinate set for
        selected atoms is returned. Reference coordinates are not included."""

        if self._indices is None:
            for conf in self._confs:
                yield conf.copy()
        else:
            indices = self._indices
            for conf in self._confs:
                yield conf[indices].copy()

    def getConformation(self, index):
        """Returns conformation at given index."""

        if self._confs is None:
            raise AttributeError('conformations are not set')
        if not isinstance(index, Integral):
            raise TypeError('index must be an integer')
        n_confs = self._n_csets
        if -n_confs <= index < n_confs:
            if index < 0:
                index = n_confs + index
            return Conformation(self, index)
        else:
            raise IndexError('conformation index out of range')

    def superpose(self, **kwargs):
        """Superpose the ensemble onto the reference coordinates.
        
        :arg ref: index of the reference coordinate. If **None**, the average 
            coordinate will be assumed as the reference. Default is **None**
        :type ref: int
        """

        ref = kwargs.pop('ref', None)
        if self._coords is None:
            raise ValueError('coordinates are not set, use `setCoords`')
        if self._confs is None or len(self._confs) == 0:
            raise ValueError('conformations are not set, use `addCoordset`')
        LOGGER.timeit('_prody_ensemble')
        self._superpose(ref=ref)  # trans kwarg is used by PDBEnsemble
        LOGGER.report('Superposition completed in %.2f seconds.',
                      '_prody_ensemble')

    def _superpose(self, **kwargs):
        """Superpose conformations and update coordinates."""

        ref = kwargs.pop('ref', None)
        quiet = kwargs.pop('quiet', False)

        indices = self._indices
        weights = self._weights
        mobs = self._confs
        if indices is None:
            idx = False
            tar = self._coords
            movs = None
        else:
            idx = True
            if self._weights is not None:
                weights = weights[indices]
            tar = self._coords[indices]
            movs = self._confs

        linalg = importLA()
        svd = linalg.svd
        det = linalg.det

        if weights is None:
            if ref is None:
                tar_com = tar.mean(0)
            else:
                tar_com = tar[ref]

            tar_org = (tar - tar_com)
            mob_org = zeros(tar_org.shape, dtype=mobs.dtype)
            tar_org = tar_org.T
        else:
            weights_sum = weights.sum()
            weights_dot = dot(weights.T, weights)
            if ref is None:
                tar_com = (tar * weights).sum(axis=0) / weights_sum
            else:
                tar_com = (tar[ref] * weights[ref]).sum(axis=0) / sum(weights[ref])
            tar_org = (tar - tar_com)
            mob_org = zeros(tar_org.shape, dtype=mobs.dtype)

        if not quiet:
            LOGGER.progress('Superposing ', len(mobs), '_prody_ensemble')
        for i, mob in enumerate(mobs):
            if idx:
                mob = mob[indices]
            if weights is None:
                mob_com = mob.mean(0)
                matrix = dot(tar_org, subtract(mob, mob_com, mob_org))
            else:
                mob_com = (mob * weights).sum(axis=0) / weights_sum
                subtract(mob, mob_com, mob_org)
                matrix = dot((tar_org * weights).T,
                             (mob_org * weights)) / weights_dot

            U, s, Vh = svd(matrix)
            Id = array([[1, 0, 0], [0, 1, 0], [0, 0, sign(det(matrix))]])
            rotation = dot(Vh.T, dot(Id, U.T))

            if movs is None:
                mobs[i] = dot(mob_org, rotation)
                add(mobs[i], tar_com, mobs[i])
            else:
                add(dot(movs[i], rotation),
                    (tar_com - dot(mob_com, rotation)), movs[i])
            if not quiet:
                LOGGER.update(i + 1, label='_prody_ensemble')
        if not quiet:
            LOGGER.finish()

    def iterpose(self, rmsd=0.0001, quiet=False):
        """Iteratively superpose the ensemble until convergence.  Initially,
        all conformations are aligned with the reference coordinates.  Then
        mean coordinates are calculated, and are set as the new reference
        coordinates.  This is repeated until reference coordinates do not
        change.  This is determined by the value of RMSD between the new and
        old reference coordinates.  Note that at the end of the iterative
        procedure the reference coordinate set will be average of conformations
        in the ensemble.

        :arg rmsd: change in reference coordinates to determine convergence,
            default is 0.0001 Å RMSD
        :type rmsd: float"""

        if self._coords is None:
            raise AttributeError('coordinates are not set, use `setCoords`')
        if self._confs is None or len(self._confs) == 0:
            raise AttributeError('conformations are not set, use'
                                 '`addCoordset`')
        LOGGER.info('Starting iterative superposition:')
        LOGGER.timeit('_prody_ensemble')
        rmsdif = 1
        step = 0
        weights = self._weights
        length = len(self)
        if weights is not None:
            if weights.ndim == 3:
                weightsum = weights.sum(axis=0)
                weightsum[weightsum==0.] = 1. # add pseudocount to avoid nan
            else:
                weightsum = length

        while rmsdif > rmsd:
            self._superpose(quiet=quiet)
            if weights is None:
                newxyz = self._confs.sum(0) / length
            else:
                newxyz = (self._confs * weights).sum(0) / weightsum
            rmsdif = getRMSD(self._coords, newxyz)
            self._coords = newxyz
            step += 1
            LOGGER.info('Step #{0}: RMSD difference = {1:.4e}'
                        .format(step, rmsdif))
        LOGGER.report('Iterative superposition completed in %.2fs.',
                      '_prody_ensemble')

    def getMSFs(self):
        """Returns mean square fluctuations (MSFs) for selected atoms.
        Conformations can be aligned using one of :meth:`superpose` or
        :meth:`iterpose` methods prior to MSF calculation."""

        if self._confs is None:
            return
        indices = self._indices
        if indices is None:
            mean = self._confs.mean(0)
            ssqf = zeros(mean.shape)
            for conf in self._confs:
                ssqf += (conf - mean) ** 2
        else:
            mean = self._confs[:, indices].mean(0)
            ssqf = zeros(mean.shape)
            for conf in self._confs[:, indices]:
                ssqf += (conf - mean) ** 2
        return ssqf.sum(1) / self._n_csets

    def getRMSFs(self):
        """Returns root mean square fluctuations (RMSFs) for selected atoms.
        Conformations can be aligned using one of :meth:`superpose` or
        :meth:`iterpose` methods prior to RMSF calculation."""

        return self.getMSFs() ** 0.5

    def getDeviations(self):
        """Returns deviations from reference coordinates for selected atoms.
        Conformations can be aligned using one of :meth:`superpose` or
        :meth:`iterpose` methods prior to calculating deviations."""

        if not isinstance(self._confs, ndarray):
            LOGGER.warning('Conformations are not set.')
            return None
        if not isinstance(self._coords, ndarray):
            LOGGER.warning('Coordinates are not set.')
            return None

        return self._getCoordsets() - self._getCoords()

    def getRMSDs(self, pairwise=False):
        """Returns root mean square deviations (RMSDs) for selected atoms.
        Conformations can be aligned using one of :meth:`superpose` or
        :meth:`iterpose` methods prior to RMSD calculation.
        
        :arg pairwise: if **True** then it will return pairwise RMSDs 
            as an n-by-n matrix. n is the number of conformations.
        :type pairwise: bool
        """

        if self._confs is None or self._coords is None:
            return None

        indices = self._indices
        if indices is None:
            indices = arange(self._confs.shape[1])
        
        weights = self._weights[indices] if self._weights is not None else None

        if pairwise:
            n_confs = self.numConfs()
            RMSDs = zeros((n_confs, n_confs))
            for i in range(n_confs):
                for j in range(i+1, n_confs):
                    RMSDs[i, j] = RMSDs[j, i] = getRMSD(self._confs[i, indices], self._confs[j, indices], weights)
        else:
            RMSDs = getRMSD(self._coords[indices], self._confs[:, indices], weights)

        return RMSDs

    def getDefvecs(self, pairwise=False):
        """Calculate and return deformation vectors (defvecs). Note that
        you might need to align the conformations using :meth:`superpose` or
        :meth:`iterpose` before calculating defvecs.

        :arg pairwise: if **True** then it will return pairwise defvecs 
            as an n-by-n matrix. n is the number of conformations.
        :type pairwise: bool
        """

        if self._confs is None or self._coords is None:
            return None

        indices = self._indices
        if indices is None:
            indices = arange(self._confs.shape[1])

        if self._weights is not None:
            if self._weights.ndim == 2:
                # Ensemble but not PDBEnsemble
                weights = tile(self._weights, (self.numConfs(), 1, 1))

            weights = self._weights[:, indices]
        else:
            weights = None

        if pairwise:
            n_confs = self.numConfs()
            defvecs = empty((n_confs, n_confs), dtype=ndarray)
            for i in range(n_confs):
                for j in range(i, n_confs):
                    if weights is None:
                        w = None
                    else:
                        wi = weights[i]; wj = weights[j]
                        w = wi * wj
                    defvecs[i, j] = defvecs[j, i] = calcDeformVector(self._confs[i, indices], 
                                                                     self._confs[j, indices], w)
        else:
            defvecs = [calcDeformVector(self._coords[indices], conf[indices], weights[i]) 
                       for i, conf in enumerate(self._confs)]

        return defvecs

    def setData(self, label, data):
        """Store atomic *data* under *label*, which must:

            * start with a letter
            * contain only alphanumeric characters and underscore
            * not be a reserved word (see :func:`.listReservedWords`)

        *data* must be a :func:`list` or a :class:`~numpy.ndarray` and its
        length must be equal to the number of atoms.  If the dimension of the
        *data* array is 1, i.e. ``data.ndim==1``, *label* may be used to make
        atom selections, e.g. ``"label 1 to 10"`` or ``"label C1 C2"``.  Note
        that, if data with *label* is present, it will be overwritten."""

        label = checkLabel(label)

        if isscalar(data):
            data = [data] * self._n_csets
            
        data = asarray(data)
        ndim, dtype, shape = data.ndim, data.dtype, data.shape

        if ndim == 1 and dtype == bool:
            raise TypeError('1 dimensional boolean arrays are not '
                            'accepted, use `setFlags` instead')

        if len(data) != self._n_csets:
            raise ValueError('len(data) must match number of coordinate sets')

        self._data[label] = data

    def delData(self, label):
        """Return data associated with *label* and remove from the instance.
        If data associated with *label* is not found, return **None**."""

        return self._data.pop(label, None)

    def getData(self, label):
        """Returns a copy of the data array associated with *label*, or **None**
        if such data is not present."""

        data = self._getData(label)
        if data is not None:
            return data.copy()

    def _getData(self, label):
        """Returns data array associated with *label*, or **None** if such data
        is not present."""

        try:
            return self._data[label]
        except KeyError:
            return None

    def isDataLabel(self, label):
        """Returns **True** if data associated with *label* is present."""

        if label in self._data:
            return True
        else:
            try:
                return self._getData(label) is not None
            except:
                return False

    def getDataLabels(self, which=None):
        """Returns data labels.  For ``which='user'``, return only labels of
        user provided data."""

        if str(which).startswith('u'):  # user
            labels = [key for key in (self._data or {})]
        else:
            labels = list(self._data or [])
        labels.sort()
        return labels

    def getDataType(self, label):
        """Returns type of the data (i.e. ``data.dtype``) associated with
        *label*, or **None** label is not used."""

        try:
            return self._data[label].dtype
        except KeyError:
            return None



# -*- coding: utf-8 -*-
"""This module defines classes measuring quantities, transforming coordinates,
and identifying contacts.


Identify contacts
=================

The following class and functions are for contact identifications:

  * :class:`.Contacts` - identify intermolecular contacts
  * :func:`.findNeighbors` - identify interacting atom pairs
  * :func:`.iterNeighbors` - identify interacting atom pairs

Measure quantities
==================

The following functions are for measuring simple quantities:

  * :func:`.calcDistance` - calculate distance(s)
  * :func:`.calcAngle` - calculate bond angle
  * :func:`.calcDihedral` - calculate dihedral angle
  * :func:`.calcOmega` - calculate omega (ω) angle
  * :func:`.calcPhi` - calculate phi (φ) angle
  * :func:`.calcPsi` - calculate psi (ψ) angle
  * :func:`.calcGyradius` - calculate radius of gyration
  * :func:`.calcCenter` - calculate geometric (or mass) center
  * :func:`.calcDeformVector` - calculate deformation vector


Anisotropic factors
===================

The following functions handle anisotropic displacement parameter (ADP) present
in some X-ray structures.

  * :func:`.buildADPMatrix` - build ADP matrix
  * :func:`.calcADPAxes` - calculate ADP axes
  * :func:`.calcADPs` - calculate ADPs

Transformations
===============

The following class and functions are for handling coordinate transformations:

  * :class:`.Transformation` - store transformation matrix
  * :func:`.alignCoordsets` - align multiple coordinate sets
  * :func:`.applyTransformation` - apply a transformation
  * :func:`.calcTransformation` - calculate a transformation
  * :func:`.calcRMSD` - calculate root-mean-square distance
  * :func:`.superpose` - superpose atoms or coordinate sets
  * :func:`.moveAtoms` - move atoms by given offset
"""

__all__ = []

from . import measure
from .measure import *
__all__.extend(measure.__all__)

from .measure import getDihedral, getPhiAtoms
from .measure import getAngle, getCenter, getCentral

from . import contacts
from .contacts import *
__all__.extend(contacts.__all__)

from . import transform
from .transform import *
__all__.extend(transform.__all__)

from .transform import getRMSD, getTransformation



"""This module contains unit tests for :mod:`~prody.proteins`."""
import numpy as np
from numpy.testing import *

from prody import *
from prody import LOGGER
from prody.tests import unittest
from prody.tests.datafiles import *

LOGGER.verbosity = 'none'


class TestParsePDBHeaderOnly(unittest.TestCase):

    def setUp(self):
        self.header = parsePDB(pathDatafile('pdb2k39_truncated.pdb'),
                               header=True, model=0)

    def testHeaderType(self):
        self.assertIsInstance(self.header, dict,
            'header type is incorrect')

    def testHeaderContent(self):
        self.assertEqual(self.header.get('classification'),
            'SIGNALING PROTEIN',
            'failed to get expected value for classification from header')
        self.assertEqual(self.header.get('experiment'), 'SOLUTION NMR',
            'failed to get expected value for experiment from header')
        self.assertEqual(self.header.get('deposition_date'), '25-APR-08',
            'failed to get expected value for deposition_date from header')
        self.assertEqual(self.header.get('identifier'), '2K39',
            'failed to get expected value for identifier from header')
        self.assertEqual(self.header.get('title'),
            'RECOGNITION DYNAMICS UP TO MICROSECONDS REVEALED FROM RDC '
            'DERIVED UBIQUITIN ENSEMBLE IN SOLUTION',
            'failed to get expected value for title from header dictionary')

    def tearDown(self):

        self.header = None

class TestParsePDBHeaderCRYST1andSCALE(unittest.TestCase):

    def setUp(self):
        self.header = parsePDB(pathDatafile('1pwc.pdb'),
                               header=True, model=0)

    def testHeaderType(self):
        self.assertIsInstance(self.header, dict,
            'header type is incorrect')

    def testHeaderContent(self):
        # CRYST1
        cryst1 = self.header.get('CRYST1', None)
        self.assertIsInstance(cryst1, dict,
            'CRYST1 type is incorrect in the header')
        self.assertEqual(cryst1.get('Z value'), 4,
            'failed to get expected value for Z value')
        self.assertEqual(cryst1.get('spaceGroup'), 'P 21 21 21',
            'failed to get expected value for spaceGroup from header')
        self.assertEqual(cryst1.get('cellAngles', None), (90.0, 90.0, 90.0),
            'failed to get expected value for cellAngles from header')
        self.assertEqual(cryst1.get('cellLength'), (50.9, 66.7, 99.6),
            'failed to get expected value for cellLength from header')

        # SCALE
        scale = self.header.get('SCALE', None)
        self.assertIsInstance(scale, dict,
            'SCALE type is incorrect in the header')
        self.assertIsInstance(scale.get('ctof', None), np.ndarray)
        self.assertEqual(scale.get('ctof').shape, (4,4), 
            'failed to get expected value for scale.ctof value')
        _ctof = np.array([[0.019646, 0.      , 0.      , 0.      ],
                          [0.      , 0.014993, 0.      , 0.      ],
                          [0.      , 0.      , 0.01004 , 0.      ],
                          [0.      , 0.      , 0.      , 1.      ]])
              
        assert_array_equal(scale.get('ctof'), _ctof, 
            'failed to get expected value for sacle.ctof value')

        self.assertIsInstance(scale.get('ftoc', None), np.ndarray)
        self.assertEqual(scale.get('ftoc').shape, (4,4), 
            'failed to get expected value for scale.ftoc value')
        _ftoc = np.array([[50.90094676,  0.        ,  0.        ,  0.        ],
                          [ 0.        , 66.6977923 ,  0.        ,  0.        ],
                          [ 0.        ,  0.        , 99.60159363,  0.        ],
                          [ 0.        ,  0.        ,  0.        ,  1.        ]])
        assert_array_almost_equal(scale.get('ftoc'), _ftoc, decimal=7,
            err_msg='failed to get expected value for scale.ftoc value')

        # REMARK 365
        self.assertIsInstance(self.header.get('missing_residues', None), list,
            'missing_residues type is incorrect in the header')
        mis_res = self.header.get('missing_residues')
        
        self.assertEqual(len(mis_res), 4, 
                         'failed to get expected value for missing_residues value')        
        self.assertEqual( mis_res[0], ('A', 'ALA', 1, '', ''),
            'failed to get expected value for missing_residues[0]')
        self.assertEqual( mis_res[1], ('A', 'ASP', 2, '', ''),
            'failed to get expected value for missing_residues[1]')
        self.assertEqual( mis_res[2], ('A', 'THR', 348, '', ''),
            'failed to get expected value for missing_residues[2]')
        self.assertEqual( mis_res[3], ('A', 'THR', 349, '', ''),
            'failed to get expected value for missing_residues[3]')
        
    def tearDown(self):

        self.header = None

class TestParsePDBHeaderMissingAtoms(unittest.TestCase):

    def setUp(self):
        self.header = parsePDB(pathDatafile('pdb3hsy.pdb'),
                               header=True, model=0)

    def testHeaderType(self):
        self.assertIsInstance(self.header, dict,
            'header type is incorrect')

    def testHeaderContent(self):
        # CRYST1
        self.assertIsInstance(self.header.get('missing_atoms', None), list,
            'missing_atoms type is incorrect in the header')
        mis_atoms = self.header.get('missing_atoms')
        
        self.assertEqual(len(mis_atoms), 5, 
                         'failed to get expected value for missing_atoms value')
        self.assertEqual( mis_atoms[0],
            ('A', 'ASN', 305, '', ['CG', 'OD1', 'ND2']), 
            'failed to get expected value for missing_atoms[0]')
        self.assertEqual( mis_atoms[1],
            ('B', 'GLN', 296, '', ['CG', 'CD', 'OE1', 'NE2']),
            'failed to get expected value for missing_atoms[1]')
        self.assertEqual( mis_atoms[2],
            ('B', 'ARG', 297, '', ['CG', 'CD', 'NE', 'CZ', 'NH1', 'NH2']),
            'failed to get expected value for missing_atoms[2]')
        self.assertEqual( mis_atoms[3],
            ('B', 'ILE', 298, '', ['CG1', 'CG2', 'CD1']),
            'failed to get expected value for missing_atoms[3]')
        self.assertEqual( mis_atoms[4],
            ('B', 'GLU', 299, '', ['CG', 'CD', 'OE1', 'OE2']),
            'failed to get expected value for missing_atoms[4]')
        
    def tearDown(self):

        self.header = None


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""This module provides utility functions and classes for handling files,
logging, type checking, etc.  Contents of this module are not included in
ProDy namespace, as it is not safe to import them all due to name conflicts.
Required or classes should be imported explicitly, e.g.
``from prody.utilities import PackageLogger, openFile``.


Package utilities
===============================================================================

  * :class:`.PackageLogger`
  * :class:`.PackageSettings`
  * :func:`.getPackagePath`
  * :func:`.setPackagePath`

Type/Value checkers
===============================================================================

  * :func:`.checkCoords`
  * :func:`.checkWeights`
  * :func:`.checkTypes`

Path/file handling
===============================================================================

  * :func:`.gunzip`
  * :func:`.openFile`
  * :func:`.openDB`
  * :func:`.openSQLite`
  * :func:`.openURL`
  * :func:`.copyFile`
  * :func:`.isExecutable`
  * :func:`.isReadable`
  * :func:`.isWritable`
  * :func:`.makePath`
  * :func:`.relpath`
  * :func:`.which`
  * :func:`.pickle`
  * :func:`.unpickle`
  * :func:`.glob`


Documentation tools
===============================================================================

  * :func:`.joinRepr`
  * :func:`.joinRepr`
  * :func:`.joinTerms`
  * :func:`.tabulate`
  * :func:`.wrapText`


Miscellaneous tools
===============================================================================

  * :func:`.rangeString`
  * :func:`.alnum`
  * :func:`.importLA`
  * :func:`.dictElement`

"""

__all__ = []

from .checkers import *
from .logger import *
from .settings import *
from .misctools import *
from .pathtools import *
from .doctools import *
from .laptools import *
from .seqtools import *
from .TreeConstruction import *
from .eigtools import *

from . import catchall
from .catchall import *
__all__.extend(catchall.__all__)



import os
import glob
import shlex
import os.path
from prody.tests import TestCase, skipIf, skipUnless
from subprocess import Popen, PIPE

from numpy.testing import *

from prody.utilities import importDec
dec = importDec()

from prody import LOGGER
LOGGER.verbosity = None

from prody.apps.prody_apps import prody_commands
from prody.tests.datafiles import TEMPDIR

from prody.tests import MATPLOTLIB, NOPRODYCMD, WINDOWS

TESTDIR = os.path.join(TEMPDIR, 'prody_tests')


class TestCommandExamples(TestCase):

    def setUp(self):

        self.cwd = os.getcwd()
        if not os.path.isdir(TESTDIR):
            os.mkdir(TESTDIR)
        os.chdir(TESTDIR)

    def tearDown(self):

        if os.path.isdir(TESTDIR):
            for fn in glob.glob(os.path.join(TESTDIR, '*')):
                os.remove(fn)
        os.chdir(self.cwd)


count = 0
for cmd in prody_commands.choices:

    parser = prody_commands.choices[cmd]

    test_examples = parser._defaults.get('test_examples', [])

    usage = parser._defaults['usage_example']

    examples = []
    for line in usage.split('\n'):
        line = line.strip()
        if line.startswith('$'):
            examples.append(line[1:].strip())

    for i in test_examples:
        try:
            egs = [examples[i]]
        except TypeError:
            egs = [examples[x] for x in i]
        count += 1

        @dec.slow
        @skipIf(NOPRODYCMD, 'prody command not found')
        @skipIf(WINDOWS, 'examples are not tested in Windows')
        def func(self, examples=egs):

            for eg in examples:
                pipe = Popen(shlex.split(eg + ' --quiet'),
                             stdout=PIPE, stderr=PIPE)
                stdout = pipe.stdout.read()
                pipe.stdout.close()
                stderr = pipe.stderr.read()
                pipe.stderr.close()

        func.__name__ = 'testCommandExample{0:d}'.format(count)
        func.__doc__ = 'Test example: $ {0:s}'.format(' $ '.join(egs))
        setattr(TestCommandExamples, func.__name__, func)

del func



# -*- coding: utf-8 -*-
"""This module defines a class and a function for rotating translating blocks
(RTB) calculations."""

import numpy as np

from prody import LOGGER
from prody.utilities import checkCoords

from .anm import ANMBase
from .gnm import GNMBase

__all__ = ['RTB']

class Increment(object):

    def __init__(self, s=0):

        self._i = s

    def __call__(self, i=1):

        self._i += i
        return self._i


class RTB(ANMBase, GNMBase):

    """Class for Rotations and Translations of Blocks (RTB) method ([FT00]_).

    .. [FT00] Tama F, Gadea FJ, Marques O, Sanejouand YH. Building-block
       approach for determining low-frequency normal modes of macromolecules.
       *Proteins* **2000** 41:1-7.

    """

    def __init__(self, name='Unknown'):

        super(RTB, self).__init__(name)
        self._project = None


    def buildHessian(self, coords, blocks, cutoff=15., gamma=1., **kwargs):
        """Build Hessian matrix for given coordinate set.

        :arg coords: a coordinate set or an object with ``getCoords`` method
        :type coords: :class:`numpy.ndarray`

        :arg blocks: a list or array of block identifiers
        :type blocks: list, :class:`numpy.ndarray`

        :arg cutoff: cutoff distance (Å) for pairwise interactions,
            default is 15.0 Å
        :type cutoff: float

        :arg gamma: spring constant, default is 1.0
        :type gamma: float
        """


        try:
            coords = (coords._getCoords() if hasattr(coords, '_getCoords') else
                      coords.getCoords())
        except AttributeError:
            try:
                checkCoords(coords)
            except TypeError:
                raise TypeError('coords must be a Numpy array or an object '
                                'with `getCoords` method')

        super(RTB, self).buildHessian(coords, cutoff=cutoff, gamma=gamma, **kwargs)

        self.calcProjection(coords, blocks, **kwargs)


    def calcProjection(self, coords, blocks, **kwargs):
        natoms = self._n_atoms

        if natoms != len(blocks):
            raise ValueError('len(blocks) must match number of atoms')

        LOGGER.timeit('_rtb')
        from collections import defaultdict
        i = Increment()
        d = defaultdict(i)
        blocks = np.array([d[b] for b in blocks], dtype='int32')

        try:
            from collections import Counter
        except ImportError:
            counter = defaultdict(int)
            for b in blocks:
                counter[b] += 1
        else:
            counter = Counter(blocks)

        nblocks = len(counter)
        maxsize = 1
        nones = 0
        while counter:
            _, size = counter.popitem()
            if size == 1:
                nones += 1
            if size > maxsize:
                maxsize = size
        LOGGER.info('System has {0} blocks, the largest with {1} of {2} units.'
                    .format(nblocks, maxsize, natoms))
        nb6 = nblocks * 6 - nones * 3

        coords = coords.T.astype(float, order='C')

        hessian = self._hessian
        self._project = project = np.zeros((natoms * 3, nb6), float)

        from .rtbtools import calc_projection

        calc_projection(coords, blocks, project, natoms, nblocks, nb6, maxsize)

        self._hessian = project.T.dot(hessian).dot(project)
        self._dof = self._hessian.shape[0]
        LOGGER.report('Block Hessian and projection matrix were calculated in %.2fs.', label='_rtb')


    def getProjection(self):
        """Returns a copy of the projection matrix."""

        if self._project is not None:
            return self._project.copy()

    def _getProjection(self):

        return self._project

    def calcModes(self, n_modes=20, zeros=False, turbo=True, **kwargs):
        """Calculate normal modes.  This method uses :func:`scipy.linalg.eigh`
        function to diagonalize the Hessian matrix. When Scipy is not found,
        :func:`numpy.linalg.eigh` is used.

        :arg n_modes: number of non-zero eigenvalues/vectors to calculate.
            If **None** is given, all modes will be calculated.
        :type n_modes: int or None, default is 20

        :arg zeros: If **True**, modes with zero eigenvalues will be kept.
        :type zeros: bool, default is **True**

        :arg turbo: Use a memory intensive, but faster way to calculate modes.
        :type turbo: bool, default is **True**
        """
        if n_modes is None:
            n_modes = self._dof
        super(RTB, self).calcModes(n_modes, zeros, turbo, **kwargs)
        self._array = np.dot(self._project, self._array)
